function [CoeffMat,numPol] = PolCoeffManyP(n,r)
% Computes the matrix associated to the operation
%   \sum {x^beta (x' P_beta x) : |beta| = r, beta_1<=beta_2<=...<=beta_n}
% 
% output
%   numPol: is the size of the set 
%       Beta = {beta: |beta| = r, beta_1<=beta_2<=...<=beta_n}
%   CoeffMat:
%       For a given P = (P_beta)_{beta \in Beta}, CoeffMat(P) gives the vector
%       of coefficients of the polynomial \sum {x^beta (x' P_beta x)} 
%       in lexicographic order

if r == 0 
    [CoeffMat,alpha] = PolCoeffOneQuadratic(n);
    numPol = 1;
else
    alphaPol = partitions(n,r); %polynomial are indexed by partitions 
    %full(alphaPol)
    numPol = size(alphaPol,1);
    
    [Coeff2,Alpha2] = PolCoeffOneQuadratic(n); 
    num2 = size(Alpha2,1);
    %full(Coeff2)
    
    CoeffMat = [];
    I = speye(numPol);
    posVec = ones(numPol,1);%position of indicator
    for i = 1:numPol
        while posVec(i) <= num2
            beta = alphaPol(i,:) + Alpha2(posVec(i),:); %alpha being added
            coeff = kron(I(i,:),Coeff2(posVec(i),:)); %coefficeint being added
            posVec(i) = posVec(i)+1;
            for j = i+1:numPol
                if   beta == alphaPol(j,:) + Alpha2(posVec(j),:); %alpha being added
                    coeff = coeff + kron(I(j,:),Coeff2(posVec(j),:)); %coefficeint being added
                    posVec(j) = posVec(j)+1;
                end
            end
            %visual(n,numPol,coeff);
            CoeffMat = [CoeffMat; sparse(coeff)];
            
        end
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
function [CoeffMat,AlphaVec] = PolCoeffOneQuadratic(n)
%Computes the coefficients of a Quadratic polynomial. ...

    CoeffMat = [];
    AlphaVec = [];
    for i=1:n
        for j = i:n 
            matij = sparse(n,n);
            matij(i,j) = 1;
            if i~=j
                matij(j,i)=1;
            end
            CoeffMat = [CoeffMat; matij(:)'];
            alpha = zeros(1,n);
            alpha(i)=1;
            alpha(j)= alpha(j) + 1;
            AlphaVec = [AlphaVec; alpha];
        end
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
function parts = partitions(n,r)
 
 if r == 1
     parts = speye(n);
 else
    parts = [];
    partsPrev = partitions(n,r-1);
    numPartsPrev = size(partsPrev,1);
    I = speye(n);
    posVec = ones(1,n); %position of indicator
    for i = 1:n
        while posVec(i) <= numPartsPrev
            alpha = partsPrev(posVec(i),:) + I(i,:); %alpha being added
            posVec(i) = posVec(i)+1;
            for j = i+1:n
                if alpha == partsPrev(posVec(j),:) + I(j,:);
                    posVec(j) = posVec(j)+1;
                end
            end
            parts = [parts; alpha];
        end
    end
 end

 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function visual(n,numPol,coef)
matCoeff = reshape(full(coef),n,n,numPol);

for i=1:numPol
   subplot(1,numPol,i);
   spy(matCoeff(:,:,i));
end

