function G = graphList(name)
% usage G = graphList(name)
% where name is one of 'G8', 'G11', 'G14', 'G17'
% returns
%    G.edges
%    G.numVert 
%    G.adj 
%    G.numEdges 

G8edges = [1 2; 1 5; 2 3; 2 7; 3 4; 3 6; 4 8; 5 6; 6 7; 7 8];
G8XY = [0 1; 2 1; 5 1; 7 1;0 0; 2 0; 5 0; 7 0];

G11edges = [G8edges; 2 9; 6 9; 9 11; 10 11; 3 10; 7 10];
G11XY = [G8XY; 1 2; 3 2 ; 2 2.5];


G14edges = [G11edges; 6 12; 2 12; 10 12; 12 14; 13 14; 3 13; 7 13];
G14XY = [G11XY; 4 2; 6 2; 5 2.5];

G17edges = [G14edges; 6 15 ; 2 15; 10 15; 15 17; 16 17; 3 16; 7 16; 12 16];
G17XY = [G8XY; 0.5 1.5; 1.5 1.5 ; 1 2 ; 5.5 1.5; 6.5 1.5; 6 2; 3 2; 4 2;3.5 2.5];

switch name
    case 'G8'
        G = graphInfo(G8edges,8,1,G8XY);
    case 'G11'
        G = graphInfo(G11edges,11,1,G11XY);
    case 'G14'
        G = graphInfo(G14edges,14,1,G14XY);
    case 'G17'
        G = graphInfo(G17edges,17,1,G17XY);
    otherwise
        fprintf('\n\nCase %s not in list\n\n',name);
        G = [];
end





%---------------------------------------------
function G = graphInfo(edges,numVert,plot,XY)
%constructs G from the edges and numVert
%if plot =1 plots the resulting graph using coordinate XY for the vertices.

G.edges = edges;
G.numVert = numVert;
G.adj = sparse(G.edges(:,1),G.edges(:,2),1,G.numVert,G.numVert);
G.adj = (G.adj + G.adj') > 0;
G.numEdges = sum(sum(G.adj));

if plot
    G.xy = XY;
    gplot(G.adj,G.xy,'b-');
    hold on
    gplot(G.adj,G.xy,'r*');
    hold off
end

