function [nu,Extrainfo,A,b,c,K] = nu_r(AdjG,r)
% computes nu_r(G), for a graph G.
% usage [nu,Extrainfo,A,b,c,K] = nu_r(BG,r)
%
% input
%    AdjG: Adjacency matrix of G
%       r: Degree of the approximation  
% output
%    nu: value of nu_r(G)
%    Extrainfo.tcons = time to construct the constraint matrix
%    Extrainfo.tSedumi = SeDuMi's cpu time 
%    Extrainfo.iterSedumi = SeDuMi's number of iterations
%    A,b,c,K: SDP formulation of nu_r(G) (in SeDuMi's format)

t0 = clock;
if sum(abs(AdjG - AdjG')) > 0
    fprintf('\nThe "adjacency matrix" is not symetric');
    return
end
AdjG = sparse(AdjG);
[n] = size(AdjG,1);

% Construct matrix L
fprintf('\n\nConstructing matrix Sim:');
[L,numPol] = PolCoeffManyP(n,r);

fprintf(' size %dx%d\n\n',size(L))

L1 = PolCoeffOneP(n,r);

t1 = clock;
Extrainfo.tcons = etime(t1,t0);

% Set up parameters for SeDuMi

firstcol = L1*AdjG(:) ;
todelete = find(firstcol> 0) ;
L(todelete,:) = [] ;
L1(todelete,:) = [] ;

BG = AdjG + speye(n) ;
    A = [-L1*BG(:) speye(size(L,1)) L]; 
    b = -L1*ones(n^2,1); 
    c = [1,zeros(1,sum(size(L)))];

    K.f = 1 ;
    K.l = size(L,1);
    K.s = n*ones(numPol,1);
    
    [nks, mks]=size(K.s);
    initialindx = K.f+K.l ;
    todelete = [] ;
    numdelcols = 0 ;
    for count=1:numPol
        initialindx = K.f+K.l + (count-1)*n^2;
        if nnz(A(:,initialindx + 1:initialindx + n^2)) == 0
            todelete = [todelete initialindx+1:initialindx+n^2] ;
            numdelcols = numdelcols + 1 ;
        end ;
    end ;
        
    A(:,todelete) = [] ;
    c(todelete) = [] ;
    K.s = n*ones(numPol-numdelcols,1);
        
[x,y,info] = sedumi(A,b,c,K) ;

if info.pinf 
    % Not done so far 
else
    % Give representation 
    nu = x(1) ;
    fprintf('\n\n\n nu_%d(G) = %f\n',r,nu);
end ;

Extrainfo.tSedumi = info.cpusec;
Extrainfo.iterSedumi = info.iter;
  

