function [th2,Extrainfo,A,b,c,K] = theta_2(AdjG)
% computes theta_2(G) for a graph G.
% usage [th2,Extrainfo,A,b,c,K] = theta_2(AdjG)
% input
%      AdjG: Adjacency matrix of G
% output
%      th2: value of theta^2(G) 
%      Extrainfo.tcons = time to construct the constraint matrix
%      Extrainfo.tSedumi = SeDuMi's cpu time 
%      Extrainfo.iterSedumi = SeDuMi's number of iterations
%      A,b,c,K: SDP formulation of nu_r(G) (in SeDuMi's format)

t0=clock;
AdjG = sparse(AdjG);
[n] = size(AdjG,1);

% Construct matrix L
fprintf('\n\nConstructing matrix Sim:');
[L,numPol] = PolCoeffManyP(n,2);

fprintf(' size %dx%d\n\n',size(L))

L1 = PolCoeffOneP(n,2);
L2 = PolCoeffTheta2Pol4(n);

t1 = clock;
Extrainfo.tcons = etime(t1,t0);

% Set up parameters for SeDuMi

firstcol = L1*AdjG(:) ;
todelete = find(firstcol> 0) ;
L(todelete,:) = [] ;
L1(todelete,:) = [] ;
L2(todelete,:) = [] ;

BG = AdjG + speye(n) ;
    A = [-L1*BG(:) speye(size(L,1)) L]; 

    b = -L1*ones(n^2,1); 
    c = [1,zeros(1,sum(size(L)))];

    K.f = 1 ;
    K.l = size(L,1);
    K.s = n*ones(numPol,1);
    
    [nks, mks]=size(K.s);
    initialindx = K.f+K.l ;
    todelete = [] ;
    numdelcols = 0 ;
    for count=1:numPol
        initialindx = K.f+K.l + (count-1)*n^2;
        if nnz(A(:,initialindx + 1:initialindx + n^2)) == 0
            todelete = [todelete initialindx+1:initialindx+n^2] ;
            numdelcols = numdelcols + 1 ;
        end ;
    end ;
    A(:,todelete) = [] ;
    c(todelete) = [] ;
    K.s = n*ones(numPol-numdelcols,1);
A = [A L2] ;
c = [c,zeros(1,n^2*(n+1)^2/4)] ;
K.s = [K.s; n*(n+1)/2] ; 

[x,y,info] = sedumi(A,b,c,K) ;


if info.pinf 
    % Not done so far 
else
    % Give representation 
    th2 = x(1) ;
    fprintf('\n\n\n theta_2(G) = %f\n',th2);
end ;

Extrainfo.tSedumi = info.cpusec;
Extrainfo.iterSedumi = info.iter;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
function [CoeffMat] = PolCoeffTheta2Pol4(n)
%Computes the matrix form for the coefficients corresponding to the degree
%4 polynomial

alpha2 = partitions(n,2);
numAlpha2 = size(alpha2,1);
    
CoeffMat = [];
I = speye(numAlpha2);
posVec = ones(numAlpha2,1);%position of indicator
for i = 1:numAlpha2
   while posVec(i) <= numAlpha2
       beta = alpha2(i,:) + alpha2(posVec(i),:); %beta being added
       coeff = sparse(i,posVec(i),1,numAlpha2,numAlpha2); %coefficient being added
       posVec(i) = posVec(i)+1;
       for j = i+1:numAlpha2
           if  beta == alpha2(j,:) + alpha2(posVec(j),:); %beta being added
               coeff(j,posVec(j)) = 1; %coefficient being added
               posVec(j) = posVec(j)+1;
           end
       end
       CoeffMat = [CoeffMat; coeff(:)'];
   end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
function parts = partitions(n,r)
 
 if r == 1
     parts = speye(n);
 else
    parts = [];
    partsPrev = partitions(n,r-1);
    numPartsPrev = size(partsPrev,1);
    I = speye(n);
    posVec = ones(1,n); %position of indicator
    for i = 1:n
        while posVec(i) <= numPartsPrev
            alpha = partsPrev(posVec(i),:) + I(i,:); %alpha being added
            posVec(i) = posVec(i)+1;
            for j = i+1:n
                if alpha == partsPrev(posVec(j),:) + I(j,:);
                    posVec(j) = posVec(j)+1;
                end
            end
            parts = [parts; alpha];
        end
    end
end


